@@ -17,17 +17,33 @@
  */
 
 #include <xfs/libxfs.h>
+#include <ctype.h>
+#ifdef ENABLE_BLKID
+#include <blkid/blkid.h>
+#else
 #include <disk/fstyp.h>
 #include <disk/volume.h>
-#include <ctype.h>
+#endif
 #include "xfs_mkfs.h"
 
+/*
+ * Device topology information.
+ */
+struct fs_topology {
+	int	dsunit;		/* stripe unit - data subvolume */
+	int	dswidth;	/* stripe width - data subvolume */
+	int	rtswidth;	/* stripe width - rt subvolume */
+	int	sectorsize;
+	int	sectoralign;
+};
+
 /*
  * Prototypes for internal functions.
  */
 static void conflict(char opt, char *tab[], int oldidx, int newidx);
 static void illegal(char *value, char *opt);
-static void reqval(char opt, char *tab[], int idx);
+static __attribute__((noreturn)) void usage (void);
+static __attribute__((noreturn)) void reqval(char opt, char *tab[], int idx);
 static void respec(char opt, char *tab[], int idx);
 static void unknown(char opt, char *s);
 static int  ispow2(unsigned int i);
@@ -266,6 +282,159 @@ calc_stripe_factors(
 	}
 }
 
+#ifdef ENABLE_BLKID
+/*
+ * Check for existing filesystem or partition table on device.
+ * Returns:
+ *	 1 for existing fs or partition
+ *	 0 for nothing found
+ *	-1 for internal error
+ */
+static int
+check_overwrite(
+	char		*device)
+{
+	const char	*type;
+	blkid_probe	pr = NULL;
+	int		ret;
+	int		fd;
+	long long	size;
+	int		bsz;
+
+	if (!device || !*device)
+		return 0;
+
+	ret = -1; /* will reset on success of all setup calls */
+
+	fd = open(device, O_RDONLY);
+	if (fd < 0)
+		goto out;
+	platform_findsizes(device, fd, &size, &bsz);
+	close(fd);
+
+	/* nothing to overwrite on a 0-length device */
+	if (size == 0) {
+		ret = 0;
+		goto out;
+	}
+
+	pr = blkid_new_probe_from_filename(device);
+	if (!pr)
+		goto out;
+
+	ret = blkid_probe_enable_partitions(pr, 1);
+	if (ret < 0)
+		goto out;
+
+	ret = blkid_do_fullprobe(pr);
+	if (ret < 0)
+		goto out;
+
+	/*
+	 * Blkid returns 1 for nothing found and 0 when it finds a signature,
+	 * but we want the exact opposite, so reverse the return value here.
+	 *
+	 * In addition print some useful diagnostics about what actually is
+	 * on the device.
+	 */
+	if (ret) {
+		ret = 0;
+		goto out;
+	}
+
+	if (!blkid_probe_lookup_value(pr, "TYPE", &type, NULL)) {
+		fprintf(stderr,
+			_("%s: %s appears to contain an existing "
+			"filesystem (%s).\n"), progname, device, type);
+	} else if (!blkid_probe_lookup_value(pr, "PTTYPE", &type, NULL)) {
+		fprintf(stderr,
+			_("%s: %s appears to contain a partition "
+			"table (%s).\n"), progname, device, type);
+	} else {
+		fprintf(stderr,
+			_("%s: %s appears to contain something weird "
+			"according to blkid\n"), progname, device);
+	}
+	ret = 1;
+
+out:
+	if (pr)
+		blkid_free_probe(pr);
+	if (ret == -1)
+		fprintf(stderr,
+			_("%s: probe of %s failed, cannot detect "
+			  "existing filesystem.\n"), progname, device);
+	return ret;
+}
+
+static void blkid_get_topology(const char *device, int *sunit, int *swidth, int *sectorsize)
+{
+	blkid_topology tp;
+	blkid_probe pr;
+	unsigned long val;
+	struct stat statbuf;
+
+	/* can't get topology info from a file */
+	if (!stat(device, &statbuf) && S_ISREG(statbuf.st_mode))
+		return;
+
+	pr = blkid_new_probe_from_filename(device);
+	if (!pr)
+		return;
+
+	tp = blkid_probe_get_topology(pr);
+	if (!tp)
+		goto out_free_probe;
+
+	/*
+	 * Blkid reports the information in terms of bytes, but we want it in
+	 * terms of 512 bytes blocks (just to convert it to bytes later..)
+	 *
+	 * If the reported values are just the normal 512 byte block size
+	 * do not bother to report anything.  It will just causes warnings
+	 * if people specifier larger stripe units or widths manually.
+	 */
+	val = blkid_topology_get_minimum_io_size(tp) >> 9;
+	if (val > 1)
+		*sunit = val;
+	val = blkid_topology_get_optimal_io_size(tp) >> 9;
+	if (val > 1)
+		*swidth = val;
+	val = blkid_probe_get_sectorsize(pr);
+	*sectorsize = val;
+
+	if (blkid_topology_get_alignment_offset(tp) != 0) {
+		fprintf(stderr,
+			_("warning: device is not properly aligned %s\n"),
+			device);
+	}
+
+	blkid_free_probe(pr);
+	return;
+
+out_free_probe:
+	blkid_free_probe(pr);
+	fprintf(stderr,
+		_("warning: unable to probe device toplology for device %s\n"),
+		device);
+}
+
+static void get_topology(libxfs_init_t *xi, struct fs_topology *ft)
+{
+	if (!xi->disfile) {
+		const char *dfile = xi->volname ? xi->volname : xi->dname;
+
+		blkid_get_topology(dfile, &ft->dsunit, &ft->dswidth,
+				   &ft->sectorsize);
+	}
+
+	if (xi->rtname && !xi->risfile) {
+		int dummy;
+
+		blkid_get_topology(xi->rtname, &dummy, &ft->rtswidth, &dummy);
+	}
+}
+#else /* ENABLE_BLKID */
 static int
 check_overwrite(
 	char		*device)
@@ -289,6 +458,36 @@ check_overwrite(
 	return 0;
 }
 
+static void get_topology(libxfs_init_t *xi, struct fs_topology *ft)
+{
+	char *dfile = xi->volname ? xi->volname : xi->dname;
+	int bsz = BBSIZE;
+
+	if (!xi->disfile) {
+		int fd;
+		long long dummy;
+
+		get_subvol_stripe_wrapper(dfile, SVTYPE_DATA,
+				&ft->dsunit, &ft->dswidth, &ft->sectoralign);
+		fd = open(dfile, O_RDONLY);
+		/* If this fails we just fall back to BBSIZE */
+		if (fd >= 0) {
+			platform_findsizes(dfile, fd, &dummy, &bsz);
+			close(fd);
+		}
+	}
+
+	ft->sectorsize = bsz;
+
+	if (xi->rtname && !xi->risfile) {
+		int dummy1;
+
+		get_subvol_stripe_wrapper(dfile, SVTYPE_RT, &dummy1,
+					  &ft->rtswidth, &dummy1);
+	}
+}
+#endif /* ENABLE_BLKID */
+
 static void
 fixup_log_stripe_unit(
 	int		lsflag,
@@ -362,13 +561,13 @@ validate_log_size(__uint64_t logblocks, int blocklog, int min_logblocks)
 	}
 	if (logblocks > XFS_MAX_LOG_BLOCKS) {
 		fprintf(stderr,
-	_("log size %lld blocks too large, maximum size is %d blocks\n"),
+	_("log size %lld blocks too large, maximum size is %lld blocks\n"),
 			(long long)logblocks, XFS_MAX_LOG_BLOCKS);
 		usage();
 	}
 	if ((logblocks << blocklog) > XFS_MAX_LOG_BYTES) {
 		fprintf(stderr,
-	_("log size %lld bytes too large, maximum size is %d bytes\n"),
+	_("log size %lld bytes too large, maximum size is %lld bytes\n"),
 			(long long)(logblocks << blocklog), XFS_MAX_LOG_BYTES);
 		usage();
 	}
@@ -424,7 +623,7 @@ calc_default_ag_geometry(
 	 * number of data blocks available, trying to keep the
 	 * number of AGs relatively small (especially compared
 	 * to the original algorithm).  AG count is calculated
-	 * based on the prefered AG size, not vice-versa - the
+	 * based on the preferred AG size, not vice-versa - the
 	 * count can be increased by growfs, so prefer to use
 	 * smaller counts at mkfs time.
 	 *
@@ -550,7 +749,7 @@ zero_old_xfs_structures(
 	xfs_off_t		off;
 
 	/*
-	 * read in existing filesystem superblock, use it's geometry
+	 * read in existing filesystem superblock, use its geometry
 	 * settings and zero the existing secondary superblocks.
 	 */
 	buf = memalign(libxfs_device_alignment(), new_sb->sb_sectsize);
@@ -567,7 +766,7 @@ zero_old_xfs_structures(
 		free(buf);
 		return;
 	}
-	libxfs_xlate_sb(buf, &sb, 1, XFS_SB_ALL_BITS);
+	libxfs_sb_from_disk(&sb, buf);
 
 	/*
 	 * perform same basic superblock validation to make sure we
@@ -604,6 +803,20 @@ done:
 	free(buf);
 }
 
+static void
+discard_blocks(dev_t dev, __uint64_t nsectors)
+{
+	int fd;
+
+	/*
+	 * We intentionally ignore errors from the discard ioctl.  It is
+	 * not necessary for the mkfs functionality but just an optimization.
+	 */
+	fd = libxfs_device_to_fd(dev);
+	if (fd > 0)
+		platform_discard_blocks(fd, 0, nsectors << 9);
+}
+
 int
 main(
 	int			argc,
@@ -616,7 +829,7 @@ main(
 	__uint64_t		agsize;
 	xfs_alloc_rec_t		*arec;
 	int			attrversion;
-	xfs_btree_sblock_t	*block;
+	struct xfs_btree_block	*block;
 	int			blflag;
 	int			blocklog;
 	unsigned int		blocksize;
@@ -680,6 +893,7 @@ main(
 	int			nvflag;
 	int			nci;
 	int			Nflag;
+	int			discard = 1;
 	char			*p;
 	char			*protofile;
 	char			*protostring;
@@ -691,7 +905,6 @@ main(
 	char			*rtfile;
 	char			*rtsize;
 	xfs_sb_t		*sbp;
-	int			sectoralign;
 	int			sectorlog;
 	unsigned int		sectorsize;
 	__uint64_t		sector_mask;
@@ -701,8 +914,7 @@ main(
 	uuid_t			uuid;
 	int			worst_freelist;
 	libxfs_init_t		xi;
-	int 			xlv_dsunit;
-	int			xlv_dswidth;
+	struct fs_topology	ft;
 	int			lazy_sb_counters;
 
 	progname = basename(argv[0]);
@@ -733,14 +945,14 @@ main(
 	nodsflag = norsflag = 0;
 	force_overwrite = 0;
 	worst_freelist = 0;
-	lazy_sb_counters = 0;
+	lazy_sb_counters = 1;
 	memset(&fsx, 0, sizeof(fsx));
 
 	memset(&xi, 0, sizeof(xi));
 	xi.isdirect = LIBXFS_DIRECT;
 	xi.isreadonly = LIBXFS_EXCLUSIVELY;
 
-	while ((c = getopt(argc, argv, "b:d:i:l:L:n:Np:qr:s:CfV")) != EOF) {
+	while ((c = getopt(argc, argv, "b:d:i:l:L:n:KNp:qr:s:CfV")) != EOF) {
 		switch (c) {
 		case 'C':
 		case 'f':
@@ -1256,9 +1468,12 @@ main(
 		case 'N':
 			Nflag = 1;
 			break;
+		case 'K':
+			discard = 0;
+			break;
 		case 'p':
 			if (protofile)
-				respec('p', 0, 0);
+				respec('p', NULL, 0);
 			protofile = optarg;
 			break;
 		case 'q':
@@ -1392,24 +1607,45 @@ main(
 		usage();
 	}
 
-	sectoralign = 0;
-	xlv_dsunit = xlv_dswidth = 0;
-	if (!xi.disfile)
-		get_subvol_stripe_wrapper(dfile, SVTYPE_DATA,
-				&xlv_dsunit, &xlv_dswidth, &sectoralign);
-	if (sectoralign) {
+	memset(&ft, 0, sizeof(ft));
+	get_topology(&xi, &ft);
+
+	if (ft.sectoralign) {
+		/*
+		 * Older Linux software RAID versions want the sector size
+		 * to match the block size to avoid switching I/O sizes.
+		 * For the legacy libdisk case we thus set the sector size to
+		 * match the block size.  For systems using libblkid we assume
+		 * that the kernel is recent enough to not require this and
+		 * ft.sectoralign will never be set.
+		 */
 		sectorsize = blocksize;
+	} else if (!ssflag) {
+		/*
+		 * Unless specified manually on the command line use the
+		 * advertised sector size of the device.
+		 */
+		sectorsize = ft.sectorsize ? ft.sectorsize : XFS_MIN_SECTORSIZE;
+	}
+
+	if (ft.sectoralign || !ssflag) {
 		sectorlog = libxfs_highbit32(sectorsize);
 		if (loginternal) {
 			lsectorsize = sectorsize;
 			lsectorlog = sectorlog;
 		}
 	}
+
 	if (sectorsize < XFS_MIN_SECTORSIZE ||
 	    sectorsize > XFS_MAX_SECTORSIZE || sectorsize > blocksize) {
 		fprintf(stderr, _("illegal sector size %d\n"), sectorsize);
 		usage();
 	}
+	if (sectorsize < ft.sectorsize) {
+		fprintf(stderr, _("illegal sector size %d; hw sector is %d\n"),
+			sectorsize, ft.sectorsize);
+		usage();
+	}
 	if (lsectorsize < XFS_MIN_SECTORSIZE ||
 	    lsectorsize > XFS_MAX_SECTORSIZE || lsectorsize > blocksize) {
 		fprintf(stderr, _("illegal log sector size %d\n"), lsectorsize);
@@ -1545,14 +1781,15 @@ main(
 		 * and the underlying volume is striped, then set rtextblocks
 		 * to the stripe width.
 		 */
-		int		dummy1, rswidth;
+		int		rswidth;
 		__uint64_t	rtextbytes;
 
-		dummy1 = rswidth = 0;
+		rswidth = 0;
 
 		if (!norsflag && !xi.risfile && !(!rtsize && xi.disfile))
-			get_subvol_stripe_wrapper(dfile, SVTYPE_RT, &dummy1,
-						  &rswidth, &dummy1);
+			rswidth = ft.rtswidth;
+		else
+			rswidth = 0;
 
 		/* check that rswidth is a multiple of fs blocksize */
 		if (!norsflag && rswidth && !(BBTOB(rswidth) % blocksize)) {
@@ -1602,10 +1839,7 @@ main(
 	calc_stripe_factors(dsu, dsw, sectorsize, lsu, lsectorsize,
 				&dsunit, &dswidth, &lsunit);
 
-	if (slflag || ssflag)
-		xi.setblksize = sectorsize;
-	else
-		xi.setblksize = 1;
+	xi.setblksize = sectorsize;
 
 	/*
 	 * Initialize.  This will open the log and rt devices as well.
@@ -1644,6 +1878,14 @@ main(
 		}
 	}
 
+	if (discard) {
+		discard_blocks(xi.ddev, xi.dsize);
+		if (xi.rtdev)
+			discard_blocks(xi.rtdev, xi.rtsize);
+		if (xi.logdev && xi.logdev != xi.ddev)
+			discard_blocks(xi.logdev, xi.logBBsize);
+	}
+
 	if (!liflag && !ldflag)
 		loginternal = xi.logdev == 0;
 	if (xi.logname)
@@ -1717,7 +1959,7 @@ reported by the device (%u).\n"),
 	min_logblocks = max_tr_res * XFS_MIN_LOG_FACTOR;
 	min_logblocks = MAX(XFS_MIN_LOG_BLOCKS, min_logblocks);
 	if (!logsize && dblocks >= (1024*1024*1024) >> blocklog)
-		min_logblocks = MAX(min_logblocks, (10*1024*1024)>>blocklog);
+		min_logblocks = MAX(min_logblocks, XFS_MIN_LOG_BYTES>>blocklog);
 	if (logsize && xi.logBBsize > 0 && logblocks > DTOBT(xi.logBBsize)) {
 		fprintf(stderr,
 _("size %s specified for log subvolume is too large, maximum is %lld blocks\n"),
@@ -1737,10 +1979,10 @@ _("size %s specified for log subvolume is too large, maximum is %lld blocks\n"),
 		logblocks = 0;
 	} else if (loginternal && !logsize) {
 		/*
-		 * logblocks grows from min_logblocks to XFS_MAX_LOG_BLOCKS
-		 * at 128GB
-		 *
-		 * 2048 = 128GB / MAX_LOG_BYTES
+		 * With a 2GB max log size, default to maximum size
+		 * at 4TB. This keeps the same ratio from the older
+		 * max log size of 128M at 256GB fs size. IOWs,
+		 * the ratio of fs size to log size is 2048:1.
 		 */
 		logblocks = (dblocks << blocklog) / 2048;
 		logblocks = logblocks >> blocklog;
@@ -1793,27 +2035,27 @@ _("size %s specified for log subvolume is too large, maximum is %lld blocks\n"),
 		agsize = dblocks / agcount + (dblocks % agcount != 0);
 	else
 		calc_default_ag_geometry(blocklog, dblocks,
-				xlv_dsunit | xlv_dswidth, &agsize, &agcount);
+				ft.dsunit | ft.dswidth, &agsize, &agcount);
 
 	if (!nodsflag) {
 		if (dsunit) {
-			if (xlv_dsunit && xlv_dsunit != dsunit) {
+			if (ft.dsunit && ft.dsunit != dsunit) {
 				fprintf(stderr,
 					_("%s: Specified data stripe unit %d "
 					"is not the same as the volume stripe "
 					"unit %d\n"),
-					progname, dsunit, xlv_dsunit);
+					progname, dsunit, ft.dsunit);
 			}
-			if (xlv_dswidth && xlv_dswidth != dswidth) {
+			if (ft.dswidth && ft.dswidth != dswidth) {
 				fprintf(stderr,
 					_("%s: Specified data stripe width %d "
 					"is not the same as the volume stripe "
 					"width %d\n"),
-					progname, dswidth, xlv_dswidth);
+					progname, dswidth, ft.dswidth);
 			}
 		} else {
-			dsunit = xlv_dsunit;
-			dswidth = xlv_dswidth;
+			dsunit = ft.dsunit;
+			dswidth = ft.dswidth;
 			nodsflag = 1;
 		}
 	} /* else dsunit & dswidth can't be set if nodsflag is set */
@@ -1845,6 +2087,7 @@ _("size %s specified for log subvolume is too large, maximum is %lld blocks\n"),
 			 */
 			if (tmp_agsize > XFS_AG_MAX_BLOCKS(blocklog))
 				tmp_agsize = ((agsize) / dsunit) * dsunit;
+
 			if ((tmp_agsize >= XFS_AG_MIN_BLOCKS(blocklog)) &&
 			    (tmp_agsize <= XFS_AG_MAX_BLOCKS(blocklog))) {
 				agsize = tmp_agsize;
@@ -1972,7 +2215,7 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 	if (loginternal) {
 		/*
 		 * Readjust the log size to fit within an AG if it was sized
-		 * automaticly.
+		 * automatically.
 		 */
 		if (!logsize) {
 			logblocks = MIN(logblocks,
@@ -2124,7 +2367,7 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 	/* OK, now write the superblock */
 	buf = libxfs_getbuf(xi.ddev, XFS_SB_DADDR, XFS_FSS_TO_BB(mp, 1));
 	memset(XFS_BUF_PTR(buf), 0, sectorsize);
-	libxfs_xlate_sb(XFS_BUF_PTR(buf), sbp, -1, XFS_SB_ALL_BITS);
+	libxfs_sb_to_disk((void *)XFS_BUF_PTR(buf), sbp, XFS_SB_ALL_BITS);
 	libxfs_writebuf(buf, LIBXFS_EXIT_ON_FAILURE);
 
 	/*
@@ -2176,7 +2419,7 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 				XFS_AG_DADDR(mp, agno, XFS_SB_DADDR),
 				XFS_FSS_TO_BB(mp, 1));
 		memset(XFS_BUF_PTR(buf), 0, sectorsize);
-		libxfs_xlate_sb(XFS_BUF_PTR(buf), sbp, -1, XFS_SB_ALL_BITS);
+		libxfs_sb_to_disk((void *)XFS_BUF_PTR(buf), sbp, XFS_SB_ALL_BITS);
 		libxfs_writebuf(buf, LIBXFS_EXIT_ON_FAILURE);
 
 		/*
@@ -2189,25 +2432,23 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 		memset(agf, 0, sectorsize);
 		if (agno == agcount - 1)
 			agsize = dblocks - (xfs_drfsbno_t)(agno * agsize);
-		INT_SET(agf->agf_magicnum, ARCH_CONVERT, XFS_AGF_MAGIC);
-		INT_SET(agf->agf_versionnum, ARCH_CONVERT, XFS_AGF_VERSION);
-		INT_SET(agf->agf_seqno, ARCH_CONVERT, agno);
-		INT_SET(agf->agf_length, ARCH_CONVERT, (xfs_agblock_t)agsize);
-		INT_SET(agf->agf_roots[XFS_BTNUM_BNOi], ARCH_CONVERT,
-				XFS_BNO_BLOCK(mp));
-		INT_SET(agf->agf_roots[XFS_BTNUM_CNTi], ARCH_CONVERT,
-				XFS_CNT_BLOCK(mp));
-		INT_SET(agf->agf_levels[XFS_BTNUM_BNOi], ARCH_CONVERT, 1);
-		INT_SET(agf->agf_levels[XFS_BTNUM_CNTi], ARCH_CONVERT, 1);
-		INT_SET(agf->agf_flfirst, ARCH_CONVERT, 0);
-		INT_SET(agf->agf_fllast, ARCH_CONVERT, XFS_AGFL_SIZE(mp) - 1);
-		INT_SET(agf->agf_flcount, ARCH_CONVERT, 0);
+		agf->agf_magicnum = cpu_to_be32(XFS_AGF_MAGIC);
+		agf->agf_versionnum = cpu_to_be32(XFS_AGF_VERSION);
+		agf->agf_seqno = cpu_to_be32(agno);
+		agf->agf_length = cpu_to_be32(agsize);
+		agf->agf_roots[XFS_BTNUM_BNOi] = cpu_to_be32(XFS_BNO_BLOCK(mp));
+		agf->agf_roots[XFS_BTNUM_CNTi] = cpu_to_be32(XFS_CNT_BLOCK(mp));
+		agf->agf_levels[XFS_BTNUM_BNOi] = cpu_to_be32(1);
+		agf->agf_levels[XFS_BTNUM_CNTi] = cpu_to_be32(1);
+		agf->agf_flfirst = 0;
+		agf->agf_fllast = cpu_to_be32(XFS_AGFL_SIZE(mp) - 1);
+		agf->agf_flcount = 0;
 		nbmblocks = (xfs_extlen_t)(agsize - XFS_PREALLOC_BLOCKS(mp));
-		INT_SET(agf->agf_freeblks, ARCH_CONVERT, nbmblocks);
-		INT_SET(agf->agf_longest, ARCH_CONVERT, nbmblocks);
+		agf->agf_freeblks = cpu_to_be32(nbmblocks);
+		agf->agf_longest = cpu_to_be32(nbmblocks);
 		if (loginternal && agno == logagno) {
-			INT_MOD(agf->agf_freeblks, ARCH_CONVERT, -logblocks);
-			INT_SET(agf->agf_longest, ARCH_CONVERT, agsize -
+			be32_add_cpu(&agf->agf_freeblks, -logblocks);
+			agf->agf_longest = cpu_to_be32(agsize -
 				XFS_FSB_TO_AGBNO(mp, logstart) - logblocks);
 		}
 		if (XFS_MIN_FREELIST(agf, mp) > worst_freelist)
@@ -2222,18 +2463,18 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 				XFS_FSS_TO_BB(mp, 1));
 		agi = XFS_BUF_TO_AGI(buf);
 		memset(agi, 0, sectorsize);
-		INT_SET(agi->agi_magicnum, ARCH_CONVERT, XFS_AGI_MAGIC);
-		INT_SET(agi->agi_versionnum, ARCH_CONVERT, XFS_AGI_VERSION);
-		INT_SET(agi->agi_seqno, ARCH_CONVERT, agno);
-		INT_SET(agi->agi_length, ARCH_CONVERT, (xfs_agblock_t)agsize);
-		INT_SET(agi->agi_count, ARCH_CONVERT, 0);
-		INT_SET(agi->agi_root, ARCH_CONVERT, XFS_IBT_BLOCK(mp));
-		INT_SET(agi->agi_level, ARCH_CONVERT, 1);
-		INT_SET(agi->agi_freecount, ARCH_CONVERT, 0);
-		INT_SET(agi->agi_newino, ARCH_CONVERT, NULLAGINO);
-		INT_SET(agi->agi_dirino, ARCH_CONVERT, NULLAGINO);
+		agi->agi_magicnum = cpu_to_be32(XFS_AGI_MAGIC);
+		agi->agi_versionnum = cpu_to_be32(XFS_AGI_VERSION);
+		agi->agi_seqno = cpu_to_be32(agno);
+		agi->agi_length = cpu_to_be32((xfs_agblock_t)agsize);
+		agi->agi_count = 0;
+		agi->agi_root = cpu_to_be32(XFS_IBT_BLOCK(mp));
+		agi->agi_level = cpu_to_be32(1);
+		agi->agi_freecount = 0;
+		agi->agi_newino = cpu_to_be32(NULLAGINO);
+		agi->agi_dirino = cpu_to_be32(NULLAGINO);
 		for (c = 0; c < XFS_AGI_UNLINKED_BUCKETS; c++)
-			INT_SET(agi->agi_unlinked[c], ARCH_CONVERT, NULLAGINO);
+			agi->agi_unlinked[c] = cpu_to_be32(NULLAGINO);
 		libxfs_writebuf(buf, LIBXFS_EXIT_ON_FAILURE);
 
 		/*
@@ -2242,48 +2483,41 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 		buf = libxfs_getbuf(mp->m_dev,
 				XFS_AGB_TO_DADDR(mp, agno, XFS_BNO_BLOCK(mp)),
 				bsize);
-		block = XFS_BUF_TO_SBLOCK(buf);
+		block = XFS_BUF_TO_BLOCK(buf);
 		memset(block, 0, blocksize);
-		INT_SET(block->bb_magic, ARCH_CONVERT, XFS_ABTB_MAGIC);
-		INT_SET(block->bb_level, ARCH_CONVERT, 0);
-		INT_SET(block->bb_numrecs, ARCH_CONVERT, 1);
-		INT_SET(block->bb_leftsib, ARCH_CONVERT, NULLAGBLOCK);
-		INT_SET(block->bb_rightsib, ARCH_CONVERT, NULLAGBLOCK);
-		arec = XFS_BTREE_REC_ADDR(blocksize, xfs_alloc, block, 1,
-			XFS_BTREE_BLOCK_MAXRECS(blocksize, xfs_alloc, 1));
-		INT_SET(arec->ar_startblock, ARCH_CONVERT,
-			XFS_PREALLOC_BLOCKS(mp));
+		block->bb_magic = cpu_to_be32(XFS_ABTB_MAGIC);
+		block->bb_level = 0;
+		block->bb_numrecs = cpu_to_be16(1);
+		block->bb_u.s.bb_leftsib = cpu_to_be32(NULLAGBLOCK);
+		block->bb_u.s.bb_rightsib = cpu_to_be32(NULLAGBLOCK);
+		arec = XFS_ALLOC_REC_ADDR(mp, block, 1);
+		arec->ar_startblock = cpu_to_be32(XFS_PREALLOC_BLOCKS(mp));
 		if (loginternal && agno == logagno) {
 			if (lalign) {
 				/*
 				 * Have to insert two records
 				 * Insert pad record for stripe align of log
 				 */
-				INT_SET(arec->ar_blockcount, ARCH_CONVERT,
-					(xfs_extlen_t)(XFS_FSB_TO_AGBNO(
-						mp, logstart)
-				  	- (INT_GET(arec->ar_startblock,
-						ARCH_CONVERT))));
+				arec->ar_blockcount = cpu_to_be32(
+					XFS_FSB_TO_AGBNO(mp, logstart) -
+					be32_to_cpu(arec->ar_startblock));
 				nrec = arec + 1;
 				/*
 				 * Insert record at start of internal log
 				 */
-				INT_SET(nrec->ar_startblock, ARCH_CONVERT,
-					INT_GET(arec->ar_startblock,
-						ARCH_CONVERT) +
-					INT_GET(arec->ar_blockcount,
-						ARCH_CONVERT));
+				nrec->ar_startblock = cpu_to_be32(
+					be32_to_cpu(arec->ar_startblock) +
+					be32_to_cpu(arec->ar_blockcount));
 				arec = nrec;
-				INT_MOD(block->bb_numrecs, ARCH_CONVERT, 1);
+				be16_add_cpu(&block->bb_numrecs, 1);
 			}
 			/*
 			 * Change record start to after the internal log
 			 */
-			INT_MOD(arec->ar_startblock, ARCH_CONVERT, logblocks);
+			be32_add_cpu(&arec->ar_startblock, logblocks);
 		}
-		INT_SET(arec->ar_blockcount, ARCH_CONVERT,
-			(xfs_extlen_t)(agsize -
-				INT_GET(arec->ar_startblock, ARCH_CONVERT)));
+		arec->ar_blockcount = cpu_to_be32(agsize - 
+					be32_to_cpu(arec->ar_startblock));
 		libxfs_writebuf(buf, LIBXFS_EXIT_ON_FAILURE);
 
 		/*
@@ -2292,35 +2526,31 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 		buf = libxfs_getbuf(mp->m_dev,
 				XFS_AGB_TO_DADDR(mp, agno, XFS_CNT_BLOCK(mp)),
 				bsize);
-		block = XFS_BUF_TO_SBLOCK(buf);
+		block = XFS_BUF_TO_BLOCK(buf);
 		memset(block, 0, blocksize);
-		INT_SET(block->bb_magic, ARCH_CONVERT, XFS_ABTC_MAGIC);
-		INT_SET(block->bb_level, ARCH_CONVERT, 0);
-		INT_SET(block->bb_numrecs, ARCH_CONVERT, 1);
-		INT_SET(block->bb_leftsib, ARCH_CONVERT, NULLAGBLOCK);
-		INT_SET(block->bb_rightsib, ARCH_CONVERT, NULLAGBLOCK);
-		arec = XFS_BTREE_REC_ADDR(blocksize, xfs_alloc, block, 1,
-			XFS_BTREE_BLOCK_MAXRECS(blocksize, xfs_alloc, 1));
-		INT_SET(arec->ar_startblock, ARCH_CONVERT,
-			XFS_PREALLOC_BLOCKS(mp));
+		block->bb_magic = cpu_to_be32(XFS_ABTC_MAGIC);
+		block->bb_level = 0;
+		block->bb_numrecs = cpu_to_be16(1);
+		block->bb_u.s.bb_leftsib = cpu_to_be32(NULLAGBLOCK);
+		block->bb_u.s.bb_rightsib = cpu_to_be32(NULLAGBLOCK);
+		arec = XFS_ALLOC_REC_ADDR(mp, block, 1);
+		arec->ar_startblock = cpu_to_be32(XFS_PREALLOC_BLOCKS(mp));
 		if (loginternal && agno == logagno) {
 			if (lalign) {
-				INT_SET(arec->ar_blockcount, ARCH_CONVERT,
-				    (xfs_extlen_t)( XFS_FSB_TO_AGBNO(
-					mp, logstart) - (INT_GET(
-					arec->ar_startblock, ARCH_CONVERT)) )
-				);
+				arec->ar_blockcount = cpu_to_be32(
+					XFS_FSB_TO_AGBNO(mp, logstart) -
+					be32_to_cpu(arec->ar_startblock));
 				nrec = arec + 1;
-				INT_SET(nrec->ar_startblock, ARCH_CONVERT,
-				    INT_GET(arec->ar_startblock, ARCH_CONVERT) +
-				    INT_GET(arec->ar_blockcount, ARCH_CONVERT));
+				nrec->ar_startblock = cpu_to_be32(
+					be32_to_cpu(arec->ar_startblock) +
+					be32_to_cpu(arec->ar_blockcount));
 				arec = nrec;
-				INT_MOD(block->bb_numrecs, ARCH_CONVERT, 1);
+				be16_add_cpu(&block->bb_numrecs, 1);
 			}
-			INT_MOD(arec->ar_startblock, ARCH_CONVERT, logblocks);
+			be32_add_cpu(&arec->ar_startblock, logblocks);
 		}
-		INT_SET(arec->ar_blockcount, ARCH_CONVERT, (xfs_extlen_t)
-			(agsize - INT_GET(arec->ar_startblock, ARCH_CONVERT)));
+		arec->ar_blockcount = cpu_to_be32(agsize - 
+					be32_to_cpu(arec->ar_startblock));
 		libxfs_writebuf(buf, LIBXFS_EXIT_ON_FAILURE);
 
 		/*
@@ -2329,13 +2559,13 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 		buf = libxfs_getbuf(mp->m_dev,
 				XFS_AGB_TO_DADDR(mp, agno, XFS_IBT_BLOCK(mp)),
 				bsize);
-		block = XFS_BUF_TO_SBLOCK(buf);
+		block = XFS_BUF_TO_BLOCK(buf);
 		memset(block, 0, blocksize);
-		INT_SET(block->bb_magic, ARCH_CONVERT, XFS_IBT_MAGIC);
-		INT_SET(block->bb_level, ARCH_CONVERT, 0);
-		INT_SET(block->bb_numrecs, ARCH_CONVERT, 0);
-		INT_SET(block->bb_leftsib, ARCH_CONVERT, NULLAGBLOCK);
-		INT_SET(block->bb_rightsib, ARCH_CONVERT, NULLAGBLOCK);
+		block->bb_magic = cpu_to_be32(XFS_IBT_MAGIC);
+		block->bb_level = 0;
+		block->bb_numrecs = 0;
+		block->bb_u.s.bb_leftsib = cpu_to_be32(NULLAGBLOCK);
+		block->bb_u.s.bb_rightsib = cpu_to_be32(NULLAGBLOCK);
 		libxfs_writebuf(buf, LIBXFS_EXIT_ON_FAILURE);
 	}
 
@@ -2373,7 +2603,7 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 		if ((c = libxfs_trans_reserve(tp, worst_freelist, 0, 0, 0, 0)))
 			res_failed(c);
 		libxfs_alloc_fix_freelist(&args, 0);
-		libxfs_trans_commit(tp, 0, NULL);
+		libxfs_trans_commit(tp, 0);
 	}
 
 	/*
@@ -2404,8 +2634,8 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 					XFS_SB_DADDR),
 				XFS_FSS_TO_BB(mp, 1),
 				LIBXFS_EXIT_ON_FAILURE);
-		INT_SET((XFS_BUF_TO_SBP(buf))->sb_rootino,
-				ARCH_CONVERT, mp->m_sb.sb_rootino);
+		XFS_BUF_TO_SBP(buf)->sb_rootino = cpu_to_be64(
+							mp->m_sb.sb_rootino);
 		libxfs_writebuf(buf, LIBXFS_EXIT_ON_FAILURE);
 		/*
 		 * and one in the middle for luck
@@ -2416,8 +2646,8 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 					XFS_SB_DADDR),
 				XFS_FSS_TO_BB(mp, 1),
 				LIBXFS_EXIT_ON_FAILURE);
-			INT_SET((XFS_BUF_TO_SBP(buf))->sb_rootino,
-				ARCH_CONVERT, mp->m_sb.sb_rootino);
+			XFS_BUF_TO_SBP(buf)->sb_rootino = cpu_to_be64(
+							mp->m_sb.sb_rootino);
 			libxfs_writebuf(buf, LIBXFS_EXIT_ON_FAILURE);
 		}
 	}
@@ -2476,7 +2706,7 @@ ispow2(
 	return (i & (i - 1)) == 0;
 }
 
-static void
+static void __attribute__((noreturn))
 reqval(
 	char		opt,
 	char		*tab[],
@@ -2566,7 +2796,7 @@ cvtnum(
 	return -1LL;
 }
 
-void
+static void __attribute__((noreturn))
 usage( void )
 {
 	fprintf(stderr, _("Usage: %s\n\
